using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DropExplorer
{
	public partial class MainForm : Form
	{
		Hashtable dropData = null;

		public MainForm()
		{
			InitializeComponent();

			linkLabel1.Left = label2.Right;
		}

		private void MainForm_DragEnter(object sender, DragEventArgs e)
		{
			e.Effect = DragDropEffects.Copy;
		}

		[System.Runtime.InteropServices.DllImport("user32.dll")]
		public static extern bool SetForegroundWindow(IntPtr hWnd);

		private void MainForm_DragDrop(object sender, DragEventArgs e)
		{
			ContentText.Text = "";
			FormatList.Items.Clear();
			dropData = new Hashtable();
			// Focus() does nothing
			SetForegroundWindow(Handle);
			foreach (string format in e.Data.GetFormats())
			{
				object data = null;
				try
				{
					//if (format == "FileContents")
					//{
					//    FileContentsDrop fcd = new FileContentsDrop(e);
					//    data = fcd.GetFiles();
					//}
					//else
					if (e.Data.GetData(format) is System.IO.MemoryStream)
					{
						System.IO.MemoryStream stream = e.Data.GetData(format) as System.IO.MemoryStream;
						data = stream.ToArray();
						//data = Encoding.Default.GetString(stream.ToArray());
						//data = Encoding.Unicode.GetString(stream.ToArray());

						//string str = "";
						//int i = stream.ReadByte();
						//int count = 0;
						//while (i != -1 && count < 1000)
						//{
						//    str += (char)i;
						//    count++;
						//    i = stream.ReadByte();
						//}
						//if (i != -1)
						//    str += "...";
						//data = str;
					}
					else
					{
						data = e.Data.GetData(format);
					}
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.ToString(), "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
				// Try to avoid the exception when dragging stuff in from Word 11:
				//MessageBox.Show(format);
				dropData.Add(format, data);
				FormatList.Items.Add(format);
			}
		}

		private void FormatList_SelectedIndexChanged(object sender, EventArgs e)
		{
			int index = FormatList.SelectedIndex;
			if (index != -1)
			{
				string format = FormatList.Items[index].ToString();
				object data = dropData[format];

				// Reset to default font
				webBrowser1.Visible = false;
				webBrowser1.DocumentText = "";
				ContentText.Text = "";
				ContentText.Font = new Font("Courier New", 9);
				ContentText.Enabled = true;
				
				if (format == "Rich Text Format" && data is string)
				{
					try
					{
						ContentText.Rtf = data as string;
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.ToString(), "RTF Exception");
					}
				}
				else if (format == "HTML Format" && data is string)
				{
					try
					{
						string strData = data as string;

						int startIndex = strData.IndexOf("<!DOCTYPE");
						if (startIndex == -1) startIndex = 0;
						strData = strData.Substring(startIndex);
						webBrowser1.Visible = true;
						webBrowser1.DocumentText = strData;
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.ToString(), "HTML Exception");
					}
				}
				else if (data is string)
				{
					ContentText.Text = data as string;
				}
				else if (data is string[])
				{
					ContentText.Text = "";
					foreach (string line in data as string[])
					{
						ContentText.Text += line + "\n";
					}
				}
				else if (data is byte[])
				{
					if (HexViewCheckbox.Checked)
					{
						StringBuilder sb = new StringBuilder();
						string hex = "";
						string raw = "";
						int pos = 0;
						foreach (byte b in data as byte[])
						{
							hex += b.ToString("X2") + " ";
							if (b < 0x20)
								raw += ".";
							else
								raw += (char) b;

							if (raw.Length == 8)
							{
								hex += " ";
								raw += " ";
							}
							if (raw.Length == 16 + 1)
							{
								sb.Append(pos.ToString("X8"));
								sb.Append("  ");
								sb.Append(hex);
								sb.Append(" ");
								sb.Append(raw);
								sb.Append(Environment.NewLine);

								hex = "";
								raw = "";
								pos += 16;
							}
						}
						if (raw.Length > 0)
						{
							sb.Append(pos.ToString("X8"));
							sb.Append("  ");
							sb.Append(hex);
							if (raw.Length < 8)
							{
								sb.Append(" ");   // Space between both 8 bytes blocks, only one space character
								raw += " ";   // Remember the space also on the "right side"
							}
							for (int i = raw.Length + 1; i <= 16 + 1; i++)
							{
								sb.Append("   ");
							}
							sb.Append(" ");
							sb.Append(raw);
						}

						ContentText.Text = sb.ToString();
					}
					else
					{
						if (UnicodeCheckbox.Checked)
							ContentText.Text = Encoding.Unicode.GetString(data as byte[]);
						else
							ContentText.Text = Encoding.Default.GetString(data as byte[]);
					}
				}
				else if (data == null)
				{
					ContentText.Text = "null";
					ContentText.Enabled = false;
				}
				else
				{
					ContentText.Text = data.GetType().ToString();
				}
			}
		}

		private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			UpdateCheck.Check("http://beta.unclassified.de/projekte/dropexplorer/update.xml");
		}

		private void linkLabel2_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			Process.Start("http://beta.unclassified.de/projekte/dropexplorer/");
		}

		private void UnicodeCheckbox_CheckedChanged(object sender, EventArgs e)
		{
			FormatList_SelectedIndexChanged(null, null);   // Update display
		}

		private void HexViewCheckbox_CheckedChanged(object sender, EventArgs e)
		{
			FormatList_SelectedIndexChanged(null, null);   // Update display
		}
	}
}